==> heptapod-rails-0.21.3.tar.gz: heptapod-rails-0.21.3/workhorse/internal/hg/hg-proxy.go <==
/*
In this file we proxy to Heptapod's internal HTTP Mercurial server (hgserve)
*/

package hg

import (
	"net/http"
	"net/http/httputil"
	"net/url"
	"regexp"
	"strconv"
	"strings"

	"gitlab.com/gitlab-org/gitlab-workhorse/internal/api"
	"gitlab.com/gitlab-org/gitlab-workhorse/internal/git"
)

const (
	PERMISSION_HEADER                  = "X-Heptapod-Permission-User"
	USERINFO_ID_HEADER                 = "X-Heptapod-UserInfo-Id"
	USERINFO_USERNAME_HEADER           = "X-Heptapod-UserInfo-Username"
	USERINFO_NAME_HEADER               = "X-Heptapod-UserInfo-Name"
	USERINFO_EMAIL_HEADER              = "X-Heptapod-UserInfo-Email"
	GL_REPOSITORY_HEADER               = "X-Heptapod-Gl-Repository"
	PROJECT_NAMESPACE_FULL_PATH_HEADER = "X-Heptapod-Project-Namespace-Full-Path"
	PROJECT_PATH_HEADER                = "X-Heptapod-Project-Path"
	HG_NATIVE_HEADER                   = "X-Heptapod-Hg-Native"
)

func BuildProxy(backend *url.URL) http.Handler {
	u := *backend
	proxy := httputil.NewSingleHostReverseProxy(&u)
	return proxy
}

func Handler(a *api.API, hgserve http.Handler) http.Handler {
	return git.ExportedPostRPCHandler(a, "hg", handleHgClosure(hgserve))
}

func alterRequest(r *http.Request, a *api.Response) {
	re := regexp.MustCompile(`.git$`)
	api_path := a.Repository.RelativePath
	r.URL.Path = re.ReplaceAllString(api_path, ".hg")

	// this is the most important header, it tells Mercurial the
	// permission level
	r.Header.Set(PERMISSION_HEADER, "heptapod-"+a.HG_ACCESS_LEVEL)

	// GL_ID being empty means it's an anonymous user (public pull)
	// In that case, we've simply never transmitted user information
	// in Heptapod.
	if len(a.GL_ID) > 0 {
		r.Header.Set(USERINFO_USERNAME_HEADER, a.GL_USERNAME)
		r.Header.Set(USERINFO_NAME_HEADER, a.HEPTAPOD_USERINFO_NAME)
		r.Header.Set(USERINFO_EMAIL_HEADER, a.HEPTAPOD_USERINFO_EMAIL)
		// TODO UserInfo-id:
		//   this is stupid: we split the symbolic id, only for
		//   it to eventually to be used in Git hooks
		//   (from py-heptapod) by prefixing with a hardcoded
		//   'user-'.
		//   We should just forward it: it's clearly designed
		//   in GitLab to be consistent across components.
		//   Fixing this requires changes that are too invasive
		//   in the Heptapod 0.12 series, but should unlock
		//   some standard GitLab features (deploy tokens?)
		split_user := strings.SplitN(a.GL_ID, "-", 2)
		if split_user[0] == "user" {
			r.Header.Set(USERINFO_ID_HEADER, split_user[1])
		}
	}
	r.Header.Set(GL_REPOSITORY_HEADER, a.GL_REPOSITORY)

	split_path := strings.Split(a.Repository.GlProjectPath, "/")
	last_segment := len(split_path) - 1
	r.Header.Set(PROJECT_NAMESPACE_FULL_PATH_HEADER, strings.Join(split_path[:last_segment], "/"))
	r.Header.Set(PROJECT_PATH_HEADER, split_path[last_segment])
	// standard formatboool gives "true" or "false", that's fine
	r.Header.Set(HG_NATIVE_HEADER, strconv.FormatBool(a.HEPTAPOD_HG_NATIVE))
}

// Use hgserve reverse proxy to handle a request, leveraging the API response
// The returned closure has the signature expected by git.postRPCHandler
func handleHgClosure(hgserve http.Handler) func(w *git.HttpResponseWriter, r *http.Request, a *api.Response) error {
	return func(w *git.HttpResponseWriter, r *http.Request, a *api.Response) error {
		// https://golang.org/pkg/net/http/#Handler says
		//     "Except for reading the body, handlers should not modify
		//      the provided Request."
		// but NewSingleHostReverseProxy is a prime counter example
		// to that, because that's exactly what its `director` does
		// An alternative would be to provide a separate `director`
		// function and span a new Proxy object for each request.
		// That would look formally better, but it's amount to the same.
		alterRequest(r, a)
		hgserve.ServeHTTP(w, r)
		return nil
	}
}

==> heptapod-rails-0.25.2.tar.gz: heptapod-rails-0.25.2/workhorse/internal/hg/hg-proxy.go <==
/*
In this file we proxy to Heptapod's internal HTTP Mercurial server (hgserve)
*/

package hg

import (
	"net/http"
	"net/http/httputil"
	"net/url"
	"regexp"
	"strconv"
	"strings"

	"gitlab.com/gitlab-org/gitlab/workhorse/internal/api"
	"gitlab.com/gitlab-org/gitlab/workhorse/internal/git"
)

const (
	PERMISSION_HEADER                  = "X-Heptapod-Permission-User"
	USERINFO_ID_HEADER                 = "X-Heptapod-UserInfo-Id"
	USERINFO_USERNAME_HEADER           = "X-Heptapod-UserInfo-Username"
	USERINFO_NAME_HEADER               = "X-Heptapod-UserInfo-Name"
	USERINFO_EMAIL_HEADER              = "X-Heptapod-UserInfo-Email"
	GL_REPOSITORY_HEADER               = "X-Heptapod-Gl-Repository"
	PROJECT_NAMESPACE_FULL_PATH_HEADER = "X-Heptapod-Project-Namespace-Full-Path"
	PROJECT_PATH_HEADER                = "X-Heptapod-Project-Path"
	HG_NATIVE_HEADER                   = "X-Heptapod-Hg-Native"
)

func BuildProxy(backend *url.URL) http.Handler {
	u := *backend
	proxy := httputil.NewSingleHostReverseProxy(&u)
	return proxy
}

func Handler(a *api.API, hgserve http.Handler) http.Handler {
	return git.ExportedPostRPCHandler(a, "hg", handleHgClosure(hgserve))
}

func alterRequest(r *http.Request, a *api.Response) {
	re := regexp.MustCompile(`.git$`)
	api_path := a.Repository.RelativePath
	r.URL.Path = re.ReplaceAllString(api_path, ".hg")

	// this is the most important header, it tells Mercurial the
	// permission level
	r.Header.Set(PERMISSION_HEADER, "heptapod-"+a.HG_ACCESS_LEVEL)

	// GL_ID being empty means it's an anonymous user (public pull)
	// In that case, we've simply never transmitted user information
	// in Heptapod.
	if len(a.GL_ID) > 0 {
		r.Header.Set(USERINFO_USERNAME_HEADER, a.GL_USERNAME)
		r.Header.Set(USERINFO_NAME_HEADER, a.HEPTAPOD_USERINFO_NAME)
		r.Header.Set(USERINFO_EMAIL_HEADER, a.HEPTAPOD_USERINFO_EMAIL)
		// TODO UserInfo-id:
		//   this is stupid: we split the symbolic id, only for
		//   it to eventually to be used in Git hooks
		//   (from py-heptapod) by prefixing with a hardcoded
		//   'user-'.
		//   We should just forward it: it's clearly designed
		//   in GitLab to be consistent across components.
		//   Fixing this requires changes that are too invasive
		//   in the Heptapod 0.12 series, but should unlock
		//   some standard GitLab features (deploy tokens?)
		split_user := strings.SplitN(a.GL_ID, "-", 2)
		if split_user[0] == "user" {
			r.Header.Set(USERINFO_ID_HEADER, split_user[1])
		}
	}
	r.Header.Set(GL_REPOSITORY_HEADER, a.GL_REPOSITORY)

	split_path := strings.Split(a.Repository.GlProjectPath, "/")
	last_segment := len(split_path) - 1
	r.Header.Set(PROJECT_NAMESPACE_FULL_PATH_HEADER, strings.Join(split_path[:last_segment], "/"))
	r.Header.Set(PROJECT_PATH_HEADER, split_path[last_segment])
	// standard formatboool gives "true" or "false", that's fine
	r.Header.Set(HG_NATIVE_HEADER, strconv.FormatBool(a.HEPTAPOD_HG_NATIVE))
}

// Use hgserve reverse proxy to handle a request, leveraging the API response
// The returned closure has the signature expected by git.postRPCHandler
func handleHgClosure(hgserve http.Handler) func(w *git.HttpResponseWriter, r *http.Request, a *api.Response) error {
	return func(w *git.HttpResponseWriter, r *http.Request, a *api.Response) error {
		// https://golang.org/pkg/net/http/#Handler says
		//     "Except for reading the body, handlers should not modify
		//      the provided Request."
		// but NewSingleHostReverseProxy is a prime counter example
		// to that, because that's exactly what its `director` does
		// An alternative would be to provide a separate `director`
		// function and span a new Proxy object for each request.
		// That would look formally better, but it's amount to the same.
		alterRequest(r, a)
		hgserve.ServeHTTP(w, r)
		return nil
	}
}
